# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class SamlNameID(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'format': 'str',
        'name_qualifier': 'str',
        'sp_name_qualifier': 'str',
        'sp_provided_id': 'str',
        'value': 'str'
    }

    attribute_map = {
        'format': 'Format',
        'name_qualifier': 'NameQualifier',
        'sp_name_qualifier': 'SPNameQualifier',
        'sp_provided_id': 'SPProvidedID',
        'value': 'Value'
    }

    def __init__(self, format=None, name_qualifier=None, sp_name_qualifier=None, sp_provided_id=None, value=None, local_vars_configuration=None):  # noqa: E501
        """SamlNameID - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._format = None
        self._name_qualifier = None
        self._sp_name_qualifier = None
        self._sp_provided_id = None
        self._value = None
        self.discriminator = None

        if format is not None:
            self.format = format
        if name_qualifier is not None:
            self.name_qualifier = name_qualifier
        if sp_name_qualifier is not None:
            self.sp_name_qualifier = sp_name_qualifier
        if sp_provided_id is not None:
            self.sp_provided_id = sp_provided_id
        if value is not None:
            self.value = value

    @property
    def format(self):
        """Gets the format of this SamlNameID.  # noqa: E501


        :return: The format of this SamlNameID.  # noqa: E501
        :rtype: str
        """
        return self._format

    @format.setter
    def format(self, format):
        """Sets the format of this SamlNameID.


        :param format: The format of this SamlNameID.  # noqa: E501
        :type format: str
        """

        self._format = format

    @property
    def name_qualifier(self):
        """Gets the name_qualifier of this SamlNameID.  # noqa: E501


        :return: The name_qualifier of this SamlNameID.  # noqa: E501
        :rtype: str
        """
        return self._name_qualifier

    @name_qualifier.setter
    def name_qualifier(self, name_qualifier):
        """Sets the name_qualifier of this SamlNameID.


        :param name_qualifier: The name_qualifier of this SamlNameID.  # noqa: E501
        :type name_qualifier: str
        """

        self._name_qualifier = name_qualifier

    @property
    def sp_name_qualifier(self):
        """Gets the sp_name_qualifier of this SamlNameID.  # noqa: E501


        :return: The sp_name_qualifier of this SamlNameID.  # noqa: E501
        :rtype: str
        """
        return self._sp_name_qualifier

    @sp_name_qualifier.setter
    def sp_name_qualifier(self, sp_name_qualifier):
        """Sets the sp_name_qualifier of this SamlNameID.


        :param sp_name_qualifier: The sp_name_qualifier of this SamlNameID.  # noqa: E501
        :type sp_name_qualifier: str
        """

        self._sp_name_qualifier = sp_name_qualifier

    @property
    def sp_provided_id(self):
        """Gets the sp_provided_id of this SamlNameID.  # noqa: E501


        :return: The sp_provided_id of this SamlNameID.  # noqa: E501
        :rtype: str
        """
        return self._sp_provided_id

    @sp_provided_id.setter
    def sp_provided_id(self, sp_provided_id):
        """Sets the sp_provided_id of this SamlNameID.


        :param sp_provided_id: The sp_provided_id of this SamlNameID.  # noqa: E501
        :type sp_provided_id: str
        """

        self._sp_provided_id = sp_provided_id

    @property
    def value(self):
        """Gets the value of this SamlNameID.  # noqa: E501


        :return: The value of this SamlNameID.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this SamlNameID.


        :param value: The value of this SamlNameID.  # noqa: E501
        :type value: str
        """

        self._value = value

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SamlNameID):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SamlNameID):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmWorkload(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'autoscaler_config': 'dict[str, object]',
        'created_dt': 'datetime',
        'distributed_number': 'int',
        'edges': 'OrmWorkloadEdges',
        'endpoints': 'dict[str, object]',
        'env_vars': 'dict[str, object]',
        'exit_code': 'int',
        'experiment_id': 'int',
        'final_cost': 'float',
        'id': 'int',
        'is_transfered': 'bool',
        'kernel_cluster_select_policies': 'dict[str, object]',
        'kubernetes_service_account': 'str',
        'last_alive_updated': 'datetime',
        'last_charged': 'datetime',
        'model_service_revision_id': 'int',
        'organization_id': 'int',
        'priority': 'int',
        'run_argument': 'dict[str, object]',
        'run_exec_id': 'int',
        'start_command': 'str',
        'status': 'str',
        'status_last_updated': 'datetime',
        'status_reason': 'str',
        'type': 'str',
        'updated_dt': 'datetime',
        'use_log_manager': 'bool',
        'version': 'int',
        'volume_mounts': 'dict[str, object]',
        'wait_kill_reason': 'str',
        'workload_kernel_cluster': 'int',
        'workload_kernel_cluster_node': 'int',
        'workload_kernel_image': 'int',
        'workload_kernel_resource_spec': 'int',
        'workspace_id': 'int'
    }

    attribute_map = {
        'autoscaler_config': 'autoscaler_config',
        'created_dt': 'created_dt',
        'distributed_number': 'distributed_number',
        'edges': 'edges',
        'endpoints': 'endpoints',
        'env_vars': 'env_vars',
        'exit_code': 'exit_code',
        'experiment_id': 'experiment_id',
        'final_cost': 'final_cost',
        'id': 'id',
        'is_transfered': 'is_transfered',
        'kernel_cluster_select_policies': 'kernel_cluster_select_policies',
        'kubernetes_service_account': 'kubernetes_service_account',
        'last_alive_updated': 'last_alive_updated',
        'last_charged': 'last_charged',
        'model_service_revision_id': 'model_service_revision_id',
        'organization_id': 'organization_id',
        'priority': 'priority',
        'run_argument': 'run_argument',
        'run_exec_id': 'run_exec_id',
        'start_command': 'start_command',
        'status': 'status',
        'status_last_updated': 'status_last_updated',
        'status_reason': 'status_reason',
        'type': 'type',
        'updated_dt': 'updated_dt',
        'use_log_manager': 'use_log_manager',
        'version': 'version',
        'volume_mounts': 'volume_mounts',
        'wait_kill_reason': 'wait_kill_reason',
        'workload_kernel_cluster': 'workload_kernel_cluster',
        'workload_kernel_cluster_node': 'workload_kernel_cluster_node',
        'workload_kernel_image': 'workload_kernel_image',
        'workload_kernel_resource_spec': 'workload_kernel_resource_spec',
        'workspace_id': 'workspace_id'
    }

    def __init__(self, autoscaler_config=None, created_dt=None, distributed_number=None, edges=None, endpoints=None, env_vars=None, exit_code=None, experiment_id=None, final_cost=None, id=None, is_transfered=None, kernel_cluster_select_policies=None, kubernetes_service_account=None, last_alive_updated=None, last_charged=None, model_service_revision_id=None, organization_id=None, priority=None, run_argument=None, run_exec_id=None, start_command=None, status=None, status_last_updated=None, status_reason=None, type=None, updated_dt=None, use_log_manager=None, version=None, volume_mounts=None, wait_kill_reason=None, workload_kernel_cluster=None, workload_kernel_cluster_node=None, workload_kernel_image=None, workload_kernel_resource_spec=None, workspace_id=None, local_vars_configuration=None):  # noqa: E501
        """OrmWorkload - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._autoscaler_config = None
        self._created_dt = None
        self._distributed_number = None
        self._edges = None
        self._endpoints = None
        self._env_vars = None
        self._exit_code = None
        self._experiment_id = None
        self._final_cost = None
        self._id = None
        self._is_transfered = None
        self._kernel_cluster_select_policies = None
        self._kubernetes_service_account = None
        self._last_alive_updated = None
        self._last_charged = None
        self._model_service_revision_id = None
        self._organization_id = None
        self._priority = None
        self._run_argument = None
        self._run_exec_id = None
        self._start_command = None
        self._status = None
        self._status_last_updated = None
        self._status_reason = None
        self._type = None
        self._updated_dt = None
        self._use_log_manager = None
        self._version = None
        self._volume_mounts = None
        self._wait_kill_reason = None
        self._workload_kernel_cluster = None
        self._workload_kernel_cluster_node = None
        self._workload_kernel_image = None
        self._workload_kernel_resource_spec = None
        self._workspace_id = None
        self.discriminator = None

        if autoscaler_config is not None:
            self.autoscaler_config = autoscaler_config
        if created_dt is not None:
            self.created_dt = created_dt
        self.distributed_number = distributed_number
        if edges is not None:
            self.edges = edges
        if endpoints is not None:
            self.endpoints = endpoints
        if env_vars is not None:
            self.env_vars = env_vars
        self.exit_code = exit_code
        self.experiment_id = experiment_id
        self.final_cost = final_cost
        if id is not None:
            self.id = id
        if is_transfered is not None:
            self.is_transfered = is_transfered
        if kernel_cluster_select_policies is not None:
            self.kernel_cluster_select_policies = kernel_cluster_select_policies
        if kubernetes_service_account is not None:
            self.kubernetes_service_account = kubernetes_service_account
        self.last_alive_updated = last_alive_updated
        self.last_charged = last_charged
        self.model_service_revision_id = model_service_revision_id
        if organization_id is not None:
            self.organization_id = organization_id
        if priority is not None:
            self.priority = priority
        if run_argument is not None:
            self.run_argument = run_argument
        self.run_exec_id = run_exec_id
        if start_command is not None:
            self.start_command = start_command
        if status is not None:
            self.status = status
        if status_last_updated is not None:
            self.status_last_updated = status_last_updated
        if status_reason is not None:
            self.status_reason = status_reason
        if type is not None:
            self.type = type
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if use_log_manager is not None:
            self.use_log_manager = use_log_manager
        if version is not None:
            self.version = version
        if volume_mounts is not None:
            self.volume_mounts = volume_mounts
        self.wait_kill_reason = wait_kill_reason
        self.workload_kernel_cluster = workload_kernel_cluster
        self.workload_kernel_cluster_node = workload_kernel_cluster_node
        if workload_kernel_image is not None:
            self.workload_kernel_image = workload_kernel_image
        self.workload_kernel_resource_spec = workload_kernel_resource_spec
        self.workspace_id = workspace_id

    @property
    def autoscaler_config(self):
        """Gets the autoscaler_config of this OrmWorkload.  # noqa: E501


        :return: The autoscaler_config of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._autoscaler_config

    @autoscaler_config.setter
    def autoscaler_config(self, autoscaler_config):
        """Sets the autoscaler_config of this OrmWorkload.


        :param autoscaler_config: The autoscaler_config of this OrmWorkload.  # noqa: E501
        :type autoscaler_config: dict[str, object]
        """

        self._autoscaler_config = autoscaler_config

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmWorkload.  # noqa: E501


        :return: The created_dt of this OrmWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmWorkload.


        :param created_dt: The created_dt of this OrmWorkload.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def distributed_number(self):
        """Gets the distributed_number of this OrmWorkload.  # noqa: E501


        :return: The distributed_number of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._distributed_number

    @distributed_number.setter
    def distributed_number(self, distributed_number):
        """Sets the distributed_number of this OrmWorkload.


        :param distributed_number: The distributed_number of this OrmWorkload.  # noqa: E501
        :type distributed_number: int
        """

        self._distributed_number = distributed_number

    @property
    def edges(self):
        """Gets the edges of this OrmWorkload.  # noqa: E501


        :return: The edges of this OrmWorkload.  # noqa: E501
        :rtype: OrmWorkloadEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmWorkload.


        :param edges: The edges of this OrmWorkload.  # noqa: E501
        :type edges: OrmWorkloadEdges
        """

        self._edges = edges

    @property
    def endpoints(self):
        """Gets the endpoints of this OrmWorkload.  # noqa: E501


        :return: The endpoints of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._endpoints

    @endpoints.setter
    def endpoints(self, endpoints):
        """Sets the endpoints of this OrmWorkload.


        :param endpoints: The endpoints of this OrmWorkload.  # noqa: E501
        :type endpoints: dict[str, object]
        """

        self._endpoints = endpoints

    @property
    def env_vars(self):
        """Gets the env_vars of this OrmWorkload.  # noqa: E501


        :return: The env_vars of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._env_vars

    @env_vars.setter
    def env_vars(self, env_vars):
        """Sets the env_vars of this OrmWorkload.


        :param env_vars: The env_vars of this OrmWorkload.  # noqa: E501
        :type env_vars: dict[str, object]
        """

        self._env_vars = env_vars

    @property
    def exit_code(self):
        """Gets the exit_code of this OrmWorkload.  # noqa: E501


        :return: The exit_code of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._exit_code

    @exit_code.setter
    def exit_code(self, exit_code):
        """Sets the exit_code of this OrmWorkload.


        :param exit_code: The exit_code of this OrmWorkload.  # noqa: E501
        :type exit_code: int
        """

        self._exit_code = exit_code

    @property
    def experiment_id(self):
        """Gets the experiment_id of this OrmWorkload.  # noqa: E501


        :return: The experiment_id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._experiment_id

    @experiment_id.setter
    def experiment_id(self, experiment_id):
        """Sets the experiment_id of this OrmWorkload.


        :param experiment_id: The experiment_id of this OrmWorkload.  # noqa: E501
        :type experiment_id: int
        """

        self._experiment_id = experiment_id

    @property
    def final_cost(self):
        """Gets the final_cost of this OrmWorkload.  # noqa: E501


        :return: The final_cost of this OrmWorkload.  # noqa: E501
        :rtype: float
        """
        return self._final_cost

    @final_cost.setter
    def final_cost(self, final_cost):
        """Sets the final_cost of this OrmWorkload.


        :param final_cost: The final_cost of this OrmWorkload.  # noqa: E501
        :type final_cost: float
        """

        self._final_cost = final_cost

    @property
    def id(self):
        """Gets the id of this OrmWorkload.  # noqa: E501


        :return: The id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmWorkload.


        :param id: The id of this OrmWorkload.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def is_transfered(self):
        """Gets the is_transfered of this OrmWorkload.  # noqa: E501


        :return: The is_transfered of this OrmWorkload.  # noqa: E501
        :rtype: bool
        """
        return self._is_transfered

    @is_transfered.setter
    def is_transfered(self, is_transfered):
        """Sets the is_transfered of this OrmWorkload.


        :param is_transfered: The is_transfered of this OrmWorkload.  # noqa: E501
        :type is_transfered: bool
        """

        self._is_transfered = is_transfered

    @property
    def kernel_cluster_select_policies(self):
        """Gets the kernel_cluster_select_policies of this OrmWorkload.  # noqa: E501


        :return: The kernel_cluster_select_policies of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._kernel_cluster_select_policies

    @kernel_cluster_select_policies.setter
    def kernel_cluster_select_policies(self, kernel_cluster_select_policies):
        """Sets the kernel_cluster_select_policies of this OrmWorkload.


        :param kernel_cluster_select_policies: The kernel_cluster_select_policies of this OrmWorkload.  # noqa: E501
        :type kernel_cluster_select_policies: dict[str, object]
        """

        self._kernel_cluster_select_policies = kernel_cluster_select_policies

    @property
    def kubernetes_service_account(self):
        """Gets the kubernetes_service_account of this OrmWorkload.  # noqa: E501


        :return: The kubernetes_service_account of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_service_account

    @kubernetes_service_account.setter
    def kubernetes_service_account(self, kubernetes_service_account):
        """Sets the kubernetes_service_account of this OrmWorkload.


        :param kubernetes_service_account: The kubernetes_service_account of this OrmWorkload.  # noqa: E501
        :type kubernetes_service_account: str
        """

        self._kubernetes_service_account = kubernetes_service_account

    @property
    def last_alive_updated(self):
        """Gets the last_alive_updated of this OrmWorkload.  # noqa: E501


        :return: The last_alive_updated of this OrmWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._last_alive_updated

    @last_alive_updated.setter
    def last_alive_updated(self, last_alive_updated):
        """Sets the last_alive_updated of this OrmWorkload.


        :param last_alive_updated: The last_alive_updated of this OrmWorkload.  # noqa: E501
        :type last_alive_updated: datetime
        """

        self._last_alive_updated = last_alive_updated

    @property
    def last_charged(self):
        """Gets the last_charged of this OrmWorkload.  # noqa: E501


        :return: The last_charged of this OrmWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._last_charged

    @last_charged.setter
    def last_charged(self, last_charged):
        """Sets the last_charged of this OrmWorkload.


        :param last_charged: The last_charged of this OrmWorkload.  # noqa: E501
        :type last_charged: datetime
        """

        self._last_charged = last_charged

    @property
    def model_service_revision_id(self):
        """Gets the model_service_revision_id of this OrmWorkload.  # noqa: E501


        :return: The model_service_revision_id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._model_service_revision_id

    @model_service_revision_id.setter
    def model_service_revision_id(self, model_service_revision_id):
        """Sets the model_service_revision_id of this OrmWorkload.


        :param model_service_revision_id: The model_service_revision_id of this OrmWorkload.  # noqa: E501
        :type model_service_revision_id: int
        """

        self._model_service_revision_id = model_service_revision_id

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmWorkload.  # noqa: E501


        :return: The organization_id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmWorkload.


        :param organization_id: The organization_id of this OrmWorkload.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def priority(self):
        """Gets the priority of this OrmWorkload.  # noqa: E501


        :return: The priority of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._priority

    @priority.setter
    def priority(self, priority):
        """Sets the priority of this OrmWorkload.


        :param priority: The priority of this OrmWorkload.  # noqa: E501
        :type priority: int
        """

        self._priority = priority

    @property
    def run_argument(self):
        """Gets the run_argument of this OrmWorkload.  # noqa: E501


        :return: The run_argument of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._run_argument

    @run_argument.setter
    def run_argument(self, run_argument):
        """Sets the run_argument of this OrmWorkload.


        :param run_argument: The run_argument of this OrmWorkload.  # noqa: E501
        :type run_argument: dict[str, object]
        """

        self._run_argument = run_argument

    @property
    def run_exec_id(self):
        """Gets the run_exec_id of this OrmWorkload.  # noqa: E501


        :return: The run_exec_id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._run_exec_id

    @run_exec_id.setter
    def run_exec_id(self, run_exec_id):
        """Sets the run_exec_id of this OrmWorkload.


        :param run_exec_id: The run_exec_id of this OrmWorkload.  # noqa: E501
        :type run_exec_id: int
        """

        self._run_exec_id = run_exec_id

    @property
    def start_command(self):
        """Gets the start_command of this OrmWorkload.  # noqa: E501


        :return: The start_command of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._start_command

    @start_command.setter
    def start_command(self, start_command):
        """Sets the start_command of this OrmWorkload.


        :param start_command: The start_command of this OrmWorkload.  # noqa: E501
        :type start_command: str
        """

        self._start_command = start_command

    @property
    def status(self):
        """Gets the status of this OrmWorkload.  # noqa: E501


        :return: The status of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this OrmWorkload.


        :param status: The status of this OrmWorkload.  # noqa: E501
        :type status: str
        """

        self._status = status

    @property
    def status_last_updated(self):
        """Gets the status_last_updated of this OrmWorkload.  # noqa: E501


        :return: The status_last_updated of this OrmWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._status_last_updated

    @status_last_updated.setter
    def status_last_updated(self, status_last_updated):
        """Sets the status_last_updated of this OrmWorkload.


        :param status_last_updated: The status_last_updated of this OrmWorkload.  # noqa: E501
        :type status_last_updated: datetime
        """

        self._status_last_updated = status_last_updated

    @property
    def status_reason(self):
        """Gets the status_reason of this OrmWorkload.  # noqa: E501


        :return: The status_reason of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._status_reason

    @status_reason.setter
    def status_reason(self, status_reason):
        """Sets the status_reason of this OrmWorkload.


        :param status_reason: The status_reason of this OrmWorkload.  # noqa: E501
        :type status_reason: str
        """

        self._status_reason = status_reason

    @property
    def type(self):
        """Gets the type of this OrmWorkload.  # noqa: E501


        :return: The type of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this OrmWorkload.


        :param type: The type of this OrmWorkload.  # noqa: E501
        :type type: str
        """

        self._type = type

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmWorkload.  # noqa: E501


        :return: The updated_dt of this OrmWorkload.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmWorkload.


        :param updated_dt: The updated_dt of this OrmWorkload.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def use_log_manager(self):
        """Gets the use_log_manager of this OrmWorkload.  # noqa: E501


        :return: The use_log_manager of this OrmWorkload.  # noqa: E501
        :rtype: bool
        """
        return self._use_log_manager

    @use_log_manager.setter
    def use_log_manager(self, use_log_manager):
        """Sets the use_log_manager of this OrmWorkload.


        :param use_log_manager: The use_log_manager of this OrmWorkload.  # noqa: E501
        :type use_log_manager: bool
        """

        self._use_log_manager = use_log_manager

    @property
    def version(self):
        """Gets the version of this OrmWorkload.  # noqa: E501


        :return: The version of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this OrmWorkload.


        :param version: The version of this OrmWorkload.  # noqa: E501
        :type version: int
        """

        self._version = version

    @property
    def volume_mounts(self):
        """Gets the volume_mounts of this OrmWorkload.  # noqa: E501


        :return: The volume_mounts of this OrmWorkload.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, volume_mounts):
        """Sets the volume_mounts of this OrmWorkload.


        :param volume_mounts: The volume_mounts of this OrmWorkload.  # noqa: E501
        :type volume_mounts: dict[str, object]
        """

        self._volume_mounts = volume_mounts

    @property
    def wait_kill_reason(self):
        """Gets the wait_kill_reason of this OrmWorkload.  # noqa: E501


        :return: The wait_kill_reason of this OrmWorkload.  # noqa: E501
        :rtype: str
        """
        return self._wait_kill_reason

    @wait_kill_reason.setter
    def wait_kill_reason(self, wait_kill_reason):
        """Sets the wait_kill_reason of this OrmWorkload.


        :param wait_kill_reason: The wait_kill_reason of this OrmWorkload.  # noqa: E501
        :type wait_kill_reason: str
        """

        self._wait_kill_reason = wait_kill_reason

    @property
    def workload_kernel_cluster(self):
        """Gets the workload_kernel_cluster of this OrmWorkload.  # noqa: E501


        :return: The workload_kernel_cluster of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._workload_kernel_cluster

    @workload_kernel_cluster.setter
    def workload_kernel_cluster(self, workload_kernel_cluster):
        """Sets the workload_kernel_cluster of this OrmWorkload.


        :param workload_kernel_cluster: The workload_kernel_cluster of this OrmWorkload.  # noqa: E501
        :type workload_kernel_cluster: int
        """

        self._workload_kernel_cluster = workload_kernel_cluster

    @property
    def workload_kernel_cluster_node(self):
        """Gets the workload_kernel_cluster_node of this OrmWorkload.  # noqa: E501


        :return: The workload_kernel_cluster_node of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._workload_kernel_cluster_node

    @workload_kernel_cluster_node.setter
    def workload_kernel_cluster_node(self, workload_kernel_cluster_node):
        """Sets the workload_kernel_cluster_node of this OrmWorkload.


        :param workload_kernel_cluster_node: The workload_kernel_cluster_node of this OrmWorkload.  # noqa: E501
        :type workload_kernel_cluster_node: int
        """

        self._workload_kernel_cluster_node = workload_kernel_cluster_node

    @property
    def workload_kernel_image(self):
        """Gets the workload_kernel_image of this OrmWorkload.  # noqa: E501


        :return: The workload_kernel_image of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._workload_kernel_image

    @workload_kernel_image.setter
    def workload_kernel_image(self, workload_kernel_image):
        """Sets the workload_kernel_image of this OrmWorkload.


        :param workload_kernel_image: The workload_kernel_image of this OrmWorkload.  # noqa: E501
        :type workload_kernel_image: int
        """

        self._workload_kernel_image = workload_kernel_image

    @property
    def workload_kernel_resource_spec(self):
        """Gets the workload_kernel_resource_spec of this OrmWorkload.  # noqa: E501


        :return: The workload_kernel_resource_spec of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._workload_kernel_resource_spec

    @workload_kernel_resource_spec.setter
    def workload_kernel_resource_spec(self, workload_kernel_resource_spec):
        """Sets the workload_kernel_resource_spec of this OrmWorkload.


        :param workload_kernel_resource_spec: The workload_kernel_resource_spec of this OrmWorkload.  # noqa: E501
        :type workload_kernel_resource_spec: int
        """

        self._workload_kernel_resource_spec = workload_kernel_resource_spec

    @property
    def workspace_id(self):
        """Gets the workspace_id of this OrmWorkload.  # noqa: E501


        :return: The workspace_id of this OrmWorkload.  # noqa: E501
        :rtype: int
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this OrmWorkload.


        :param workspace_id: The workspace_id of this OrmWorkload.  # noqa: E501
        :type workspace_id: int
        """

        self._workspace_id = workspace_id

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmWorkload):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmWorkload):
            return True

        return self.to_dict() != other.to_dict()



import sys
import time
import warnings
from typing import Dict, List

from vessl.openapi_client import (
    ModelServiceGatewayUpdateAPIInput,
    ModelserviceModelServiceListResponse,
    ModelserviceModelServiceRevisionListResponse,
    ModelServiceRevisionCreateFromYAMLAPIInput,
    ModelServiceRevisionUpdateAPIInput,
    ModelServiceRolloutCreateAPIInput,
)
from vessl.openapi_client import OrmModelServiceGatewayTrafficSplitEntry as TrafficSplitEntry
from vessl.openapi_client import (
    ResponseModelServiceGatewayInfo,
    ResponseModelServiceInfo,
    ResponseModelServiceRevision,
    ResponseModelServiceRolloutInfo,
    ResponseSimpleModelServiceRevision,
    V1Autoscaling,
    V1RunArgument
)
from vessl.openapi_client.models.model_service_revision_create_from_yamlv2_api_input import (
    ModelServiceRevisionCreateFromYAMLV2APIInput,
)
from vessl import vessl_api
from vessl.util.deprecaate import deprecated
from vessl.util.echo import print_warning, print_info, print_success, print_error
from vessl.util.exception import VesslApiException


def list_services(organization: str) -> ModelserviceModelServiceListResponse:
    """Get a list of all services in an organization

    Args:
        organization(str): The name of the organization.

    Example:
        ```python
        vessl.list_services(organization="my-org")
        ```
    """
    return vessl_api.model_service_list_api(organization_name=organization)

def _read_service(service_name: str) -> ResponseModelServiceInfo:
    """Get a service from a service name.

    Args:
        service_name(str): The name of the service.

    Example:
        ```python
        vessl.read_service(service_name="my-service")
        ```
    """
    return vessl_api.model_service_read_api(
        organization_name=vessl_api.organization.name, model_service_name=service_name
        )

def create_revision_from_yaml(
    organization: str, yaml_body: str
) -> ResponseModelServiceRevision:
    """Create a new revision of service from a YAML file.

    The name of the service should be provided in the YAML in `name:`.

    Args:
        organization (str): The name of the organization.
        yaml_body (str): The YAML body of the service revision.
            It should not be a structured (i.e. list, dict, ...) object, but
            a raw string in YAML.

    Example:
        ```python
        vessl.create_revision_from_yaml(
            organization="my-org",
            yaml_body=yaml_body)
        ```
    """
    payload = ModelServiceRevisionCreateFromYAMLAPIInput(yaml_body)

    return vessl_api.model_service_revision_create_from_yamlapi(
        organization_name=organization,
        model_service_revision_create_from_yamlapi_input=payload,
    )

def create_revision_from_yaml_v2(
    organization: str, service_name: str, yaml_body: str, serverless: bool, arguments: V1RunArgument
) -> ResponseModelServiceRevision:
    """Create a new revision of service from a YAML file.

    Args:
        organization (str): The name of the organization.
        service_name (str): The name of the service to create this revision in.
        yaml_body (str): The YAML body of the service revision.
            It should not be a structured (i.e. list, dict, ...) object, but
            a raw string in YAML.
        serverless  (bool): Whether this revision is in serverless mode.
        arguments (V1RunArgument): User inputs received from prompts.

    Example:
        ```python
        vessl.create_revision_from_yaml(
            organization="my-org",
            service_name="my-svc",
            yaml_body=yaml_body)
        ```
    """
    payload = ModelServiceRevisionCreateFromYAMLV2APIInput(
        serverless=serverless, yaml_spec=yaml_body, arguments=arguments)

    return vessl_api.model_service_revision_create_from_yamlv2_api(
        organization_name=organization,
        model_service_name=service_name,
        model_service_revision_create_from_yamlv2_api_input=payload,
    )


def launch_revision(organization: str, service_name: str, revision_number: int):
    """Launch a service revision from a service name and revision number.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the modelservice.
        revision_number(int): The revision number of the modelservice.

    Example:
        ```python
        vessl.launch_revision(
            organization="my-org",
            service_name="my-service",
            revision_number=1)
        ```
    """
    vessl_api.model_service_revision_launch_api(
        organization_name=organization,
        model_service_name=service_name,
        revision_number=revision_number,
    )

@deprecated(target_command="read_revision", new_command="read_service")
def read_revision(
    organization: str, service_name: str, revision_number: int
) -> ResponseModelServiceRevision:
    """Get a service revision from a service name and revision number.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.
        revision_number(int): The revision number of the service.

    Example:
        ```python
        vessl.read_revision(
            organization="my-org",
            service_name="my-service",
            revision_number=1)
        ```
    """
    return vessl_api.model_service_revision_read_api(
        organization_name=organization,
        model_service_name=service_name,
        revision_number=revision_number,
    )

def terminate_revision(organization: str, service_name: str, revision_number: int):
    """
    Terminate a service revision from a service name and revision number.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.
        revision_number(int): The revision number of the service.

    Example:
        ```python
        vessl.terminate_revision(
            organization="my-org",
            service_name="my-service",
            revision_number=1)
        ```
    """
    return vessl_api.model_service_revision_terminate_api(
        organization_name=organization,
        model_service_name=service_name,
        revision_number=revision_number,
    )


def update_revision_autoscaler_config(
    organization: str,
    service_name: str,
    revision_number: int,
    autoscaling: V1Autoscaling,
):
    """
    Update the autoscaler config of a service revision from a service name and revision number.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.
        revision_number(int): The revision number of the service.
        autoscaling(V1Autoscaling): The autoscaler config of the service(servev1/Autoscaling).

    Example:
        ```python
        vessl.update_revision_autoscaler_config(
            organization="my-org",
            service_name="my-service",
            revision_number=1,
            auto_scaler_config=V1Autoscaling(
                min=1,
                max=2,
                metric="cpu",
                target=80,
            ))
        ```
    """
    return vessl_api.model_service_revision_update_api(
        organization_name=organization,
        model_service_name=service_name,
        revision_number=revision_number,
        model_service_revision_update_api_input=ModelServiceRevisionUpdateAPIInput(
            autoscaling=autoscaling,
        ),
    )

def update_revision_autoscaling_v2(
    organization: str,
    service_name: str,
    revision_number: int,
    autoscaling: V1Autoscaling,
):
    """
    Update the autoscaler config of a serving revision from a serving name and revision number.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the serving.
        revision_number(int): The revision number of the serving.
        autoscaling(V1Autoscaling): The autoscaler config of the serving.

    Example:
        ```python
        vessl.update_revision_autoscaler_config(
            organization="my-org",
            service_name="my-service",
            revision_number=1,
            autoscaling=V1Autoscaling(
                min=1,
                max=2,
                metric=cpu, [cpu,memory,nvidia.com/gpu]
                target=60
            ))
        ```
    """
    return vessl_api.model_service_revision_update_api(
        organization_name=organization,
        model_service_name=service_name,
        revision_number=revision_number,
        model_service_revision_update_api_input=ModelServiceRevisionUpdateAPIInput(
            autoscaling=autoscaling,
        ),
    )

def read_service(service_name: str):
    """
    Reads a service, including information about its revisions and gateway configurations.

    Args:
        service_name(str): The name of the service.

    Example:
        ```python
        vessl.read_service(service_name="my-service")
        ```
    """
    service_info = vessl_api.model_service_read_api(
        organization_name=vessl_api.organization.name,
        model_service_name=service_name
    )
    revision_info = vessl_api.model_service_revision_list_api(
        organization_name=vessl_api.organization.name,
        model_service_name=service_name
    )
    result = {
        "model_service": service_info,
        "revisions": revision_info.results
    }
    return result


@deprecated(target_command="list_revisions", new_command="read_service")
def list_revisions(
    organization: str, service_name: str
) -> List[ResponseSimpleModelServiceRevision]:
    """Get a list of all revisions of a service.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.

    Examples:
        ```python
        vessl.list_revisions(
            organization="my-org",
            service_name="my-service")
        ```
    """
    resp: ModelserviceModelServiceRevisionListResponse = vessl_api.model_service_revision_list_api(
        organization_name=organization,
        model_service_name=service_name,
    )
    return resp.results

@deprecated(target_command="read_gateway", new_command="read_service")
def read_gateway(organization: str, service_name: str) -> ResponseModelServiceGatewayInfo:
    """Get the gateway of a service.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.

    Examples:
        ```python
        vessl.read_gateway(
            organization="my-org",
            service_name="my-service")
        ```
    """
    model_service: ResponseModelServiceInfo = vessl_api.model_service_read_api(
        model_service_name=service_name,
        organization_name=organization,
    )
    return model_service.gateway_config


def update_gateway(
    organization: str, service_name: str, gateway: ModelServiceGatewayUpdateAPIInput
) -> ResponseModelServiceGatewayInfo:
    """Update the gateway of a service.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.
        gateway(ModelServiceGatewayUpdateAPIInput): The gateway of the service.

    Examples:
        ```python
        from openapi_client import ModelServiceGatewayUpdateAPIInput
        from openapi_client import OrmModelServiceGatewayTrafficSplitEntry

        gateway = ModelServiceGatewayUpdateAPIInput(
            enabled=True,
            ingress_host="my-endpoint",
            traffic_split=[
                OrmModelServiceGatewayTrafficSplitEntry(
                    revision_number=1,
                    port=2222,
                    traffic_weight=100,
                )
            ],
        )

        vessl.update_gateway(
            organization="my-org",
            service_name="my-service",
            gateway=gateway)
        ```
    """
    return vessl_api.model_service_gateway_update_api(
        model_service_name=service_name,
        organization_name=organization,
        model_service_gateway_update_api_input=gateway,
    )


def _get_updated_traffic_split_rule(
    rules_current: List[TrafficSplitEntry], revision_number: int, weight: int
) -> List[TrafficSplitEntry]:
    """
    Combines the previous traffic split rule with new rule.
    When filling the remaining weight, this function uses the one with higher revision number.

    For example, with the current rule of:
    - revision #2 (port 2222) 70%
    - revision #3 (port 3333) 30%

    with a call to this function with:
    - revision #4 (port 4444) 50%

    yields a new rule of:
    - revision #4 (port 4444) 50%
    - revision #3 (port 3333) 30%
    - revision #2 (port 2222) 20%

    Revision #3 takes priority over #2, because it has the higher number (3 > 2).
    """
    # Sort from latest revision (with highest number) to oldest
    rules_current = sorted(rules_current, key=lambda x: x.revision_number, reverse=True)

    rules_new: List[TrafficSplitEntry] = [
        TrafficSplitEntry(revision_number=revision_number, traffic_weight=weight)
    ]

    weight_remaining = 100 - weight

    # Iterate through current traffic rules and add them if possible
    for rule in rules_current:
        if weight_remaining <= 0:
            break
        new_weight = min(weight_remaining, rule.traffic_weight)
        rules_new.append(
            TrafficSplitEntry(
                revision_number=rule.revision_number, traffic_weight=new_weight
            )
        )
        weight_remaining -= new_weight
        if weight_remaining <= 0:
            break

    if weight_remaining > 0:
        # This can happen if rules_current's weight do not sum up to 100
        # (this is possible for disabled gateways).
        # Handle this case safely by delegating all remaining weights to our target rule.
        rules_new[0].traffic_weight += weight_remaining

    return rules_new


def update_gateway_for_revision(
    organization: str,
    service_name: str,
    revision_number: int,
    weight: int,
) -> ResponseModelServiceGatewayInfo:
    """Update the current gateway of a service for a specific revision.

    Args:
        organization(str): The name of the organization.
        service_name(str): The name of the service.
        revision_number(int): The revision number of the service.
        weight(int): The weight of the traffic will be distributed to revision_number.

    Examples:
        ```python
        vessl.update_gateway_for_revision(
            organization="my-org",
            service_name="my-service",
            revision_number=1,
            weight=100)
        ```
    """
    gateway_current = _read_service(service_name=service_name).gateway_config
    rules_new = _get_updated_traffic_split_rule(
        rules_current=gateway_current.rules or [],
        revision_number=revision_number,
        weight=weight,
    )

    gateway_updated = vessl_api.model_service_gateway_update_api(
        organization_name=organization,
        model_service_name=service_name,
        model_service_gateway_update_api_input=ModelServiceGatewayUpdateAPIInput(
            enabled=True,
            ingress_host=gateway_current.endpoint,
            ingress_class=gateway_current.ingress_class,
            annotations=gateway_current.annotations,
            traffic_split=rules_new,
        ),
    )
    return gateway_updated


def create_active_revision_replacement_rollout(
    organization: str, model_service_name: str, desired_active_revisions_to_weight_map: Dict[int, int],
) -> ResponseModelServiceRolloutInfo:
    """
    Create a rollout to replace the active revisions of a service with the desired revisions.
    
    Args:
        organization(str): The name of the organization.
        model_service_name(str): The name of the model service.
        desired_active_revisions_to_weight_map(dict[int, int]): A dictionary of revision numbers and their desired
        traffic weight.
    """

    revisions = []
    gateway_targets = []
    for number, weight in desired_active_revisions_to_weight_map.items():
        revisions.append({
            'type': 'number',
            'number': number,
        })
        gateway_targets.append({
            'revision': {
                'type': 'number',
                'number': number,
            },
            'traffic_weight': weight,
        })

    rollout_input = ModelServiceRolloutCreateAPIInput(
        message=f'{vessl_api.user.email} initiated a rollout',
        rollout_spec={
            'steps': [
                {
                    'step_type': 'ensure_revisions_up_and_running',
                    'ensure_revisions_up_and_running': {
                        'revisions': revisions
                    },
                },
                {
                    'step_type': 'update_endpoint',
                    'update_endpoint': {
                        'annotations': {},
                        'targets': gateway_targets,
                    },
                },
                {
                    'step_type': 'terminate_revisions',
                    'terminate_revisions': {
                        'excluded_revisions': revisions,
                    },
                },
            ],
        },
    )
    return vessl_api.model_service_rollout_create_api(
        organization_name=organization,
        model_service_name=model_service_name,
        model_service_rollout_create_api_input=rollout_input
    )


def update_model_service(service: ResponseModelServiceInfo, number: List[int], weight: List[int]):
    if len(number) == 0 or len(weight) == 0:
        print_error("Please enter target revisions' number and traffic weight.")
        return
    if len(number) != len(weight) and len(number) != 1:
        print_error("Please enter traffic weight for each revision or specify only one revision.")
        return
    if len(weight) != 0 and sum(weight) != 100:
        print_error("Sum of each revision's traffic weight should be exactly 100.")
        return

    if len(number) == 1 and len(weight) == 0:
        weight = [100]

    weight_map = dict()
    for revision_numer, traffic_weight in zip(number, weight):
        weight_map[revision_numer] = traffic_weight

    try:
        create_active_revision_replacement_rollout(
            organization=vessl_api.organization.name,
            model_service_name=service.name,
            desired_active_revisions_to_weight_map=weight_map,
        )
        print_success(f"Successfully requested updating service {service.name}.\n")
        return
    except VesslApiException as e:
        print_error(f"Failed to update service {service.name}: {e.message}")
        sys.exit(1)

def read_rollout(service_name: str, rollout_num: int) -> ResponseModelServiceRolloutInfo:
    return vessl_api.model_service_rollout_read_api(
        organization_name=vessl_api.organization.name,
        model_service_name=service_name,
        rollout_number=rollout_num,
    )

def _request_abort_rollout(service: ResponseModelServiceInfo):
    resp = vessl_api.model_service_rollout_abort_api(
        organization_name=vessl_api.organization.name,
        model_service_name=service.name,
    )
    return resp


def abort_in_progress_rollout(service: ResponseModelServiceInfo) -> bool:
    resp = _request_abort_rollout(service)
    if resp.rollback_requested:
        print_success("Current update aborted. Rollback is requested.\n")
        return True
    else:
        print_warning("Current update aborted.")
        print_warning("Could not determine the original status. Rollback is not requested.\n")
        print_warning(f"Please check the status of the service and the gateway at: {WEB_HOST}/{vessl_api.organization.name}/services/{service.name}\n")
        return False

def abort_in_progress_rollout_by_name(service_name: str) -> bool:
    service = _read_service(service_name)
    return abort_in_progress_rollout(service)

def trigger_abort_and_wait(service_name: str,  max_timeout_sec: int = 8*60, print_output: bool = False) -> bool:
    '''
    Trigger the abort of the current rollout and wait for the rollout to be aborted.

    Args:
        service_name(str): The name of the service.
        max_timeout_sec(int): The maximum time to wait for the rollout to be aborted.
        print_output(bool): Whether to print the output.

    Returns:
        bool: True if the rollout is successfully aborted, False otherwise.

    Example:
        ```python
        trigger_abort_and_wait(service_name="my-service", max_timeout_sec=480, print_output=True)
        ```

    '''

    print_info_or_skip = print_info if print_output else lambda x: None
    print_error_or_skip = print_error if print_output else lambda x: None
    try:
        service = _read_service(service_name)
        resp = _request_abort_rollout(service)
        print_info_or_skip("Aborting the existing rollout...")
        rollout_abort_timeout = max_timeout_sec
        while rollout_abort_timeout > 0:
            aborting_rollout = read_rollout(service_name, resp.target_rollout_number)
            if aborting_rollout.status != "rolling_out" and aborting_rollout.status != "ready":
                print_info_or_skip("Successfully aborted the rollout.")
                break
            rollout_abort_timeout -= 5
            time.sleep(5)
            print_info_or_skip(f">> Waiting for {max_timeout_sec - rollout_abort_timeout} seconds...")
        if rollout_abort_timeout <= 0:
            print_error_or_skip("Rollout abort timeout.")
            return False
        return True
    except VesslApiException as e:
        print_error_or_skip(f"Failed to abort rollout: {e.message}")
        return False

def _wait_for_revision_to_launch(service_name: str, revision_number: int, max_timeout_sec: int = 8*60, print_output: bool = False) -> bool:
    print_info_or_skip = print_info if print_output else lambda x: None
    print_error_or_skip = print_error if print_output else lambda x: None

    revision_launch_timeout = max_timeout_sec
    print_info_or_skip(f">> Wait for the revision to be ready...")
    while revision_launch_timeout > 0:
        revision = vessl_api.model_service_revision_read_api(
        organization_name=vessl_api.organization.name,
        model_service_name=service_name,
        revision_number=revision_number,
        )
        if revision.status == "running":
            print_info_or_skip(f"Successfully created revision in service {service_name}.\n")
            break
        revision_launch_timeout -= 5
        time.sleep(5)
        print_info_or_skip(f">> Waiting for {max_timeout_sec - revision_launch_timeout} seconds...")
    if revision_launch_timeout <= 0:
        print_error_or_skip("Revision launch timeout.")
        return False

    return True

def _wait_for_gateway_enabled(gateway: ResponseModelServiceGatewayInfo, service_name: str, max_timeout_sec: int = 8*60, print_output: bool = False) -> bool:
    print_info_or_skip = print_info if print_output else lambda x: None
    print_error_or_skip = print_error if print_output else lambda x: None
    def _check_gateway_enabled(gateway):
        return gateway.enabled and gateway.status == "success" and gateway.endpoint is not None
    if not _check_gateway_enabled(gateway):
        print_info_or_skip("Endpoint update in progress. Please wait a moment.")
        gateway_update_timeout = max_timeout_sec
        print_info_or_skip(f">> Wait for the endpoint to be ready...")
        while gateway_update_timeout > 0:
            gateway = _read_service(service_name=service_name).gateway_config
            if _check_gateway_enabled(gateway):
                break
            gateway_update_timeout -= 5
            time.sleep(5)
            print_info_or_skip(f">> Waiting for {max_timeout_sec - gateway_update_timeout} seconds...")
    
        if gateway_update_timeout <= 0:
            print_error_or_skip("Endpoint update timeout. Please check the status of the endpoint.")
            return False
    return True

def get_recent_rollout(service_name: str) -> ResponseModelServiceRolloutInfo:
    resp = vessl_api.model_service_rollout_list_api(
            organization_name=vessl_api.organization.name,
            model_service_name=service_name,
        )
    recent_rollout = resp.rollouts[0] if resp.rollouts else None
    return recent_rollout



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmLLMVectorDBConnection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'auth_header_key': 'str',
        'authentication_type': 'str',
        'created_by_id': 'int',
        'created_dt': 'datetime',
        'db_type': 'str',
        'edges': 'OrmLLMVectorDBConnectionEdges',
        'encrypted_auth_header_value': 'str',
        'encrypted_bearer_token': 'str',
        'encrypted_password': 'str',
        'host': 'str',
        'id': 'int',
        'name': 'str',
        'organization_id': 'int',
        'port': 'str',
        'slug': 'str',
        'updated_dt': 'datetime',
        'username': 'str'
    }

    attribute_map = {
        'auth_header_key': 'auth_header_key',
        'authentication_type': 'authentication_type',
        'created_by_id': 'created_by_id',
        'created_dt': 'created_dt',
        'db_type': 'db_type',
        'edges': 'edges',
        'encrypted_auth_header_value': 'encrypted_auth_header_value',
        'encrypted_bearer_token': 'encrypted_bearer_token',
        'encrypted_password': 'encrypted_password',
        'host': 'host',
        'id': 'id',
        'name': 'name',
        'organization_id': 'organization_id',
        'port': 'port',
        'slug': 'slug',
        'updated_dt': 'updated_dt',
        'username': 'username'
    }

    def __init__(self, auth_header_key=None, authentication_type=None, created_by_id=None, created_dt=None, db_type=None, edges=None, encrypted_auth_header_value=None, encrypted_bearer_token=None, encrypted_password=None, host=None, id=None, name=None, organization_id=None, port=None, slug=None, updated_dt=None, username=None, local_vars_configuration=None):  # noqa: E501
        """OrmLLMVectorDBConnection - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._auth_header_key = None
        self._authentication_type = None
        self._created_by_id = None
        self._created_dt = None
        self._db_type = None
        self._edges = None
        self._encrypted_auth_header_value = None
        self._encrypted_bearer_token = None
        self._encrypted_password = None
        self._host = None
        self._id = None
        self._name = None
        self._organization_id = None
        self._port = None
        self._slug = None
        self._updated_dt = None
        self._username = None
        self.discriminator = None

        if auth_header_key is not None:
            self.auth_header_key = auth_header_key
        if authentication_type is not None:
            self.authentication_type = authentication_type
        if created_by_id is not None:
            self.created_by_id = created_by_id
        if created_dt is not None:
            self.created_dt = created_dt
        if db_type is not None:
            self.db_type = db_type
        if edges is not None:
            self.edges = edges
        if encrypted_auth_header_value is not None:
            self.encrypted_auth_header_value = encrypted_auth_header_value
        if encrypted_bearer_token is not None:
            self.encrypted_bearer_token = encrypted_bearer_token
        if encrypted_password is not None:
            self.encrypted_password = encrypted_password
        if host is not None:
            self.host = host
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if organization_id is not None:
            self.organization_id = organization_id
        if port is not None:
            self.port = port
        if slug is not None:
            self.slug = slug
        if updated_dt is not None:
            self.updated_dt = updated_dt
        if username is not None:
            self.username = username

    @property
    def auth_header_key(self):
        """Gets the auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._auth_header_key

    @auth_header_key.setter
    def auth_header_key(self, auth_header_key):
        """Sets the auth_header_key of this OrmLLMVectorDBConnection.


        :param auth_header_key: The auth_header_key of this OrmLLMVectorDBConnection.  # noqa: E501
        :type auth_header_key: str
        """

        self._auth_header_key = auth_header_key

    @property
    def authentication_type(self):
        """Gets the authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._authentication_type

    @authentication_type.setter
    def authentication_type(self, authentication_type):
        """Sets the authentication_type of this OrmLLMVectorDBConnection.


        :param authentication_type: The authentication_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :type authentication_type: str
        """

        self._authentication_type = authentication_type

    @property
    def created_by_id(self):
        """Gets the created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._created_by_id

    @created_by_id.setter
    def created_by_id(self, created_by_id):
        """Sets the created_by_id of this OrmLLMVectorDBConnection.


        :param created_by_id: The created_by_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type created_by_id: int
        """

        self._created_by_id = created_by_id

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The created_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmLLMVectorDBConnection.


        :param created_dt: The created_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def db_type(self):
        """Gets the db_type of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The db_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._db_type

    @db_type.setter
    def db_type(self, db_type):
        """Sets the db_type of this OrmLLMVectorDBConnection.


        :param db_type: The db_type of this OrmLLMVectorDBConnection.  # noqa: E501
        :type db_type: str
        """

        self._db_type = db_type

    @property
    def edges(self):
        """Gets the edges of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The edges of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: OrmLLMVectorDBConnectionEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmLLMVectorDBConnection.


        :param edges: The edges of this OrmLLMVectorDBConnection.  # noqa: E501
        :type edges: OrmLLMVectorDBConnectionEdges
        """

        self._edges = edges

    @property
    def encrypted_auth_header_value(self):
        """Gets the encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_auth_header_value

    @encrypted_auth_header_value.setter
    def encrypted_auth_header_value(self, encrypted_auth_header_value):
        """Sets the encrypted_auth_header_value of this OrmLLMVectorDBConnection.


        :param encrypted_auth_header_value: The encrypted_auth_header_value of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_auth_header_value: str
        """

        self._encrypted_auth_header_value = encrypted_auth_header_value

    @property
    def encrypted_bearer_token(self):
        """Gets the encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_bearer_token

    @encrypted_bearer_token.setter
    def encrypted_bearer_token(self, encrypted_bearer_token):
        """Sets the encrypted_bearer_token of this OrmLLMVectorDBConnection.


        :param encrypted_bearer_token: The encrypted_bearer_token of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_bearer_token: str
        """

        self._encrypted_bearer_token = encrypted_bearer_token

    @property
    def encrypted_password(self):
        """Gets the encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._encrypted_password

    @encrypted_password.setter
    def encrypted_password(self, encrypted_password):
        """Sets the encrypted_password of this OrmLLMVectorDBConnection.


        :param encrypted_password: The encrypted_password of this OrmLLMVectorDBConnection.  # noqa: E501
        :type encrypted_password: str
        """

        self._encrypted_password = encrypted_password

    @property
    def host(self):
        """Gets the host of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The host of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """Sets the host of this OrmLLMVectorDBConnection.


        :param host: The host of this OrmLLMVectorDBConnection.  # noqa: E501
        :type host: str
        """

        self._host = host

    @property
    def id(self):
        """Gets the id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmLLMVectorDBConnection.


        :param id: The id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The name of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OrmLLMVectorDBConnection.


        :param name: The name of this OrmLLMVectorDBConnection.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def organization_id(self):
        """Gets the organization_id of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The organization_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: int
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrmLLMVectorDBConnection.


        :param organization_id: The organization_id of this OrmLLMVectorDBConnection.  # noqa: E501
        :type organization_id: int
        """

        self._organization_id = organization_id

    @property
    def port(self):
        """Gets the port of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The port of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._port

    @port.setter
    def port(self, port):
        """Sets the port of this OrmLLMVectorDBConnection.


        :param port: The port of this OrmLLMVectorDBConnection.  # noqa: E501
        :type port: str
        """

        self._port = port

    @property
    def slug(self):
        """Gets the slug of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The slug of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._slug

    @slug.setter
    def slug(self, slug):
        """Sets the slug of this OrmLLMVectorDBConnection.


        :param slug: The slug of this OrmLLMVectorDBConnection.  # noqa: E501
        :type slug: str
        """

        self._slug = slug

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmLLMVectorDBConnection.


        :param updated_dt: The updated_dt of this OrmLLMVectorDBConnection.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    @property
    def username(self):
        """Gets the username of this OrmLLMVectorDBConnection.  # noqa: E501


        :return: The username of this OrmLLMVectorDBConnection.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this OrmLLMVectorDBConnection.


        :param username: The username of this OrmLLMVectorDBConnection.  # noqa: E501
        :type username: str
        """

        self._username = username

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmLLMVectorDBConnection):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmLLMVectorDBConnection):
            return True

        return self.to_dict() != other.to_dict()



import os
from typing import Callable, List, Optional

import boto3
import botocore.client
import botocore.config

from vessl.util.common import safe_cast
from vessl.util.path import ensure_parent_dir_exists

from .abstract_volume import AbstractVolume
from .file import VolumeFile


class S3Volume(AbstractVolume):
    def __init__(
        self,
        aws_access_key_id: str = "",
        aws_secret_access_key: str = "",
        aws_session_token: Optional[str] = None,
        bucket_name: str = "",
        base_path: str = "",
        region: Optional[str] = None,
        endpoint_url: Optional[str] = None,
        force_path_style: Optional[bool] = None, # optional for backward compatibility
        verify_tls: bool = True,
    ):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.aws_session_token = aws_session_token
        self.bucket_name = bucket_name
        self.base_path = base_path
        self.region = region
        self.endpoint_url = endpoint_url
        self.force_path_style = force_path_style
        self.verify_tls = verify_tls
        self._init_s3()

    def _init_s3(self):
        cfg = botocore.config.Config(region_name=self.region or "ap-northeast-2",
                                     s3={"addressing_style": "path" if self.force_path_style else "auto"})

        s3_client = boto3.client(
            "s3",
            config=cfg,
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            aws_session_token=self.aws_session_token,
            endpoint_url=self.endpoint_url,
            verify=self.verify_tls,
        )
        self.s3_client = s3_client

    def list(self, path: str) -> List[VolumeFile]:
        key = os.path.join(self.base_path, path)

        volume_files = []
        paginator = self.s3_client.get_paginator("list_objects_v2")
        for response in paginator.paginate(Bucket=self.bucket_name, Prefix=key):
            if "Contents" not in response:
                continue

            for item in response["Contents"]:
                if item["Key"].endswith("/"):
                    continue

                volume_files.append(
                    VolumeFile(path=os.path.relpath(item["Key"], key), size=item["Size"])
                )

        return volume_files

    def download_file(
        self,
        file: VolumeFile,
        destination: str,
        progress_callback: Optional[Callable[[int], None]] = None,
    ):
        current_bytes = 0
        if progress_callback is not None:

            def _callback(bytes_added):
                nonlocal current_bytes
                current_bytes += bytes_added
                progress_callback(current_bytes)
        else:
            _callback = None

        key = os.path.join(self.base_path, file.path)

        ensure_parent_dir_exists(destination)

        self.s3_client.download_file(
            Bucket=self.bucket_name,
            Key=key,
            Filename=destination,
            Callback=_callback,
        )

    def download_directory(
        self,
        directory: str,
        destination: str,
        progress_callback: Optional[Callable[[int, int, int, int, int], None]] = None,
    ):
        return self._download_directory_serially(directory, destination, progress_callback)

    def upload_file(
        self,
        local_file: str,
        destination: str,
        progress_callback: Optional[Callable[[int], None]] = None,
    ):
        current_bytes = 0
        if progress_callback is not None:

            def _callback(bytes_added):
                nonlocal current_bytes
                current_bytes += bytes_added
                progress_callback(current_bytes)
        else:
            _callback = None

        key = os.path.join(self.base_path, destination)

        self.s3_client.upload_file(
            Filename=local_file, Bucket=self.bucket_name, Key=key, Callback=_callback
        )

    def delete_file(self, path: str):
        key = os.path.join(self.base_path, path)
        self.s3_client.delete_object(
            Bucket=self.bucket_name,
            Key=key,
        )

    def delete_directory(self, path: str):
        key = os.path.join(self.base_path, path)
        files = self.list(path)
        self.s3_client.delete_objects(
            Bucket=self.bucket_name,
            Delete={"Objects": [{"Key": os.path.join(key, v.path)} for v in files]},
        )



import sys

import click

import vessl
from vessl.cli._base import VesslGroup, vessl_argument
from vessl.cli._util import (
    print_data,
    print_table,
)
from vessl.util.prompt import prompt_choices, generic_prompter
from vessl.util.echo import print_info, print_success, print_error
from vessl.util.endpoint import Endpoint
from vessl.organization import (
    create_organization,
    list_organizations,
    read_organization,
)
from vessl.util.exception import InvalidOrganizationError


def organization_name_prompter(ctx: click.Context, param: click.Parameter, value: str) -> str:
    organizations = list_organizations()
    return prompt_choices("Organization", [x.name for x in organizations])


@click.command(name="organization", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=organization_name_prompter,
)
def read(name: str):
    organization = read_organization(organization_name=name)
    print_data(
        {
            "Name": organization.name,
            "Description": organization.description,
        }
    )


@cli.vessl_command()
def list():
    organizations = list_organizations()
    print_table(
        organizations,
        ["Name"],
        lambda x: [x.name],
    )


@cli.vessl_command()
@vessl_argument(
    "name",
    type=click.STRING,
    required=True,
    prompter=generic_prompter("Organization name"),
)
def create(name: str):
    organization = create_organization( organization_name=name)
    print_success(
        f"Created '{organization.name}'.\n"
        f"For more info: {Endpoint.organization.format(organization.name)}"
    )


def organization_name_callback(ctx: click.Context, param: click.Parameter, value: str) -> str:
    try:
        organization_name = vessl.vessl_api.set_organization(value)
        print_info(f"Organization: {organization_name}")
    except InvalidOrganizationError:
        print_error("Invalid organization. Please choose an organization using `vessl configure`.")
        sys.exit(1)


# Ensure this is called before other options with `is_eager=True` for
# other callbacks that need organization to be preconfigured.
organization_name_option = click.option(
    "--organization",
    "organization_name",
    type=click.STRING,
    hidden=True,
    is_eager=True,
    expose_value=False,
    callback=organization_name_callback,
    help="Override default organization.",
)



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class ModelServiceRevisionLaunchAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'abort_ongoing_rollouts': 'bool',
        'force': 'bool',
        'replace': 'bool',
        'replace_strategy': 'str'
    }

    attribute_map = {
        'abort_ongoing_rollouts': 'abort_ongoing_rollouts',
        'force': 'force',
        'replace': 'replace',
        'replace_strategy': 'replace_strategy'
    }

    def __init__(self, abort_ongoing_rollouts=None, force=None, replace=None, replace_strategy=None, local_vars_configuration=None):  # noqa: E501
        """ModelServiceRevisionLaunchAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._abort_ongoing_rollouts = None
        self._force = None
        self._replace = None
        self._replace_strategy = None
        self.discriminator = None

        if abort_ongoing_rollouts is not None:
            self.abort_ongoing_rollouts = abort_ongoing_rollouts
        self.force = force
        if replace is not None:
            self.replace = replace
        if replace_strategy is not None:
            self.replace_strategy = replace_strategy

    @property
    def abort_ongoing_rollouts(self):
        """Gets the abort_ongoing_rollouts of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501


        :return: The abort_ongoing_rollouts of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._abort_ongoing_rollouts

    @abort_ongoing_rollouts.setter
    def abort_ongoing_rollouts(self, abort_ongoing_rollouts):
        """Sets the abort_ongoing_rollouts of this ModelServiceRevisionLaunchAPIInput.


        :param abort_ongoing_rollouts: The abort_ongoing_rollouts of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :type abort_ongoing_rollouts: bool
        """

        self._abort_ongoing_rollouts = abort_ongoing_rollouts

    @property
    def force(self):
        """Gets the force of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501


        :return: The force of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._force

    @force.setter
    def force(self, force):
        """Sets the force of this ModelServiceRevisionLaunchAPIInput.


        :param force: The force of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :type force: bool
        """

        self._force = force

    @property
    def replace(self):
        """Gets the replace of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501


        :return: The replace of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :rtype: bool
        """
        return self._replace

    @replace.setter
    def replace(self, replace):
        """Sets the replace of this ModelServiceRevisionLaunchAPIInput.


        :param replace: The replace of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :type replace: bool
        """

        self._replace = replace

    @property
    def replace_strategy(self):
        """Gets the replace_strategy of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501


        :return: The replace_strategy of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._replace_strategy

    @replace_strategy.setter
    def replace_strategy(self, replace_strategy):
        """Sets the replace_strategy of this ModelServiceRevisionLaunchAPIInput.


        :param replace_strategy: The replace_strategy of this ModelServiceRevisionLaunchAPIInput.  # noqa: E501
        :type replace_strategy: str
        """
        allowed_values = ["", "blue-green", "clean-launch"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and replace_strategy not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `replace_strategy` ({0}), must be one of {1}"  # noqa: E501
                .format(replace_strategy, allowed_values)
            )

        self._replace_strategy = replace_strategy

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelServiceRevisionLaunchAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ModelServiceRevisionLaunchAPIInput):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class WorkspaceUpdateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_node_ids': 'list[int]',
        'idle_shutdown_minutes': 'int',
        'init_script': 'str',
        'max_running_hours': 'int',
        'name': 'str',
        'ports': 'list[OrmWorkspacePort]',
        'resource_spec': 'OrmKernelResourceSpecField',
        'resource_spec_id': 'int',
        'service_account_name': 'str',
        'volume_v2_mounts': 'dict[str, V1MountForm]',
        'volumes': 'OrmVolumeMountRequests'
    }

    attribute_map = {
        'cluster_node_ids': 'cluster_node_ids',
        'idle_shutdown_minutes': 'idle_shutdown_minutes',
        'init_script': 'init_script',
        'max_running_hours': 'max_running_hours',
        'name': 'name',
        'ports': 'ports',
        'resource_spec': 'resource_spec',
        'resource_spec_id': 'resource_spec_id',
        'service_account_name': 'service_account_name',
        'volume_v2_mounts': 'volume_v2_mounts',
        'volumes': 'volumes'
    }

    def __init__(self, cluster_node_ids=None, idle_shutdown_minutes=None, init_script=None, max_running_hours=None, name=None, ports=None, resource_spec=None, resource_spec_id=None, service_account_name=None, volume_v2_mounts=None, volumes=None, local_vars_configuration=None):  # noqa: E501
        """WorkspaceUpdateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_node_ids = None
        self._idle_shutdown_minutes = None
        self._init_script = None
        self._max_running_hours = None
        self._name = None
        self._ports = None
        self._resource_spec = None
        self._resource_spec_id = None
        self._service_account_name = None
        self._volume_v2_mounts = None
        self._volumes = None
        self.discriminator = None

        if cluster_node_ids is not None:
            self.cluster_node_ids = cluster_node_ids
        self.idle_shutdown_minutes = idle_shutdown_minutes
        self.init_script = init_script
        self.max_running_hours = max_running_hours
        self.name = name
        if ports is not None:
            self.ports = ports
        if resource_spec is not None:
            self.resource_spec = resource_spec
        self.resource_spec_id = resource_spec_id
        self.service_account_name = service_account_name
        if volume_v2_mounts is not None:
            self.volume_v2_mounts = volume_v2_mounts
        if volumes is not None:
            self.volumes = volumes

    @property
    def cluster_node_ids(self):
        """Gets the cluster_node_ids of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The cluster_node_ids of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: list[int]
        """
        return self._cluster_node_ids

    @cluster_node_ids.setter
    def cluster_node_ids(self, cluster_node_ids):
        """Sets the cluster_node_ids of this WorkspaceUpdateAPIInput.


        :param cluster_node_ids: The cluster_node_ids of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type cluster_node_ids: list[int]
        """

        self._cluster_node_ids = cluster_node_ids

    @property
    def idle_shutdown_minutes(self):
        """Gets the idle_shutdown_minutes of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The idle_shutdown_minutes of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._idle_shutdown_minutes

    @idle_shutdown_minutes.setter
    def idle_shutdown_minutes(self, idle_shutdown_minutes):
        """Sets the idle_shutdown_minutes of this WorkspaceUpdateAPIInput.


        :param idle_shutdown_minutes: The idle_shutdown_minutes of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type idle_shutdown_minutes: int
        """

        self._idle_shutdown_minutes = idle_shutdown_minutes

    @property
    def init_script(self):
        """Gets the init_script of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The init_script of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._init_script

    @init_script.setter
    def init_script(self, init_script):
        """Sets the init_script of this WorkspaceUpdateAPIInput.


        :param init_script: The init_script of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type init_script: str
        """

        self._init_script = init_script

    @property
    def max_running_hours(self):
        """Gets the max_running_hours of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The max_running_hours of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._max_running_hours

    @max_running_hours.setter
    def max_running_hours(self, max_running_hours):
        """Sets the max_running_hours of this WorkspaceUpdateAPIInput.


        :param max_running_hours: The max_running_hours of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type max_running_hours: int
        """

        self._max_running_hours = max_running_hours

    @property
    def name(self):
        """Gets the name of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The name of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this WorkspaceUpdateAPIInput.


        :param name: The name of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type name: str
        """

        self._name = name

    @property
    def ports(self):
        """Gets the ports of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The ports of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: list[OrmWorkspacePort]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this WorkspaceUpdateAPIInput.


        :param ports: The ports of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type ports: list[OrmWorkspacePort]
        """

        self._ports = ports

    @property
    def resource_spec(self):
        """Gets the resource_spec of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The resource_spec of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: OrmKernelResourceSpecField
        """
        return self._resource_spec

    @resource_spec.setter
    def resource_spec(self, resource_spec):
        """Sets the resource_spec of this WorkspaceUpdateAPIInput.


        :param resource_spec: The resource_spec of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type resource_spec: OrmKernelResourceSpecField
        """

        self._resource_spec = resource_spec

    @property
    def resource_spec_id(self):
        """Gets the resource_spec_id of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The resource_spec_id of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._resource_spec_id

    @resource_spec_id.setter
    def resource_spec_id(self, resource_spec_id):
        """Sets the resource_spec_id of this WorkspaceUpdateAPIInput.


        :param resource_spec_id: The resource_spec_id of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type resource_spec_id: int
        """

        self._resource_spec_id = resource_spec_id

    @property
    def service_account_name(self):
        """Gets the service_account_name of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The service_account_name of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._service_account_name

    @service_account_name.setter
    def service_account_name(self, service_account_name):
        """Sets the service_account_name of this WorkspaceUpdateAPIInput.


        :param service_account_name: The service_account_name of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type service_account_name: str
        """

        self._service_account_name = service_account_name

    @property
    def volume_v2_mounts(self):
        """Gets the volume_v2_mounts of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The volume_v2_mounts of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: dict[str, V1MountForm]
        """
        return self._volume_v2_mounts

    @volume_v2_mounts.setter
    def volume_v2_mounts(self, volume_v2_mounts):
        """Sets the volume_v2_mounts of this WorkspaceUpdateAPIInput.


        :param volume_v2_mounts: The volume_v2_mounts of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type volume_v2_mounts: dict[str, V1MountForm]
        """

        self._volume_v2_mounts = volume_v2_mounts

    @property
    def volumes(self):
        """Gets the volumes of this WorkspaceUpdateAPIInput.  # noqa: E501


        :return: The volumes of this WorkspaceUpdateAPIInput.  # noqa: E501
        :rtype: OrmVolumeMountRequests
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this WorkspaceUpdateAPIInput.


        :param volumes: The volumes of this WorkspaceUpdateAPIInput.  # noqa: E501
        :type volumes: OrmVolumeMountRequests
        """

        self._volumes = volumes

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkspaceUpdateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorkspaceUpdateAPIInput):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmModelServiceRevisionDeploymentSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'auto_scaler_config': 'OrmAutoscalerConfig',
        'cluster_node_ids': 'list[int]',
        'docker_credentials_id': 'int',
        'env_vars': 'list[OrmEnvVar]',
        'image_url': 'str',
        'ports': 'list[OrmWorkloadPort]',
        'resource_spec': 'OrmKernelResourceSpecField',
        'resource_spec_id': 'int',
        'service_account_name': 'str',
        'start_command': 'str',
        'volume_mount_requests': 'OrmVolumeMountRequests'
    }

    attribute_map = {
        'auto_scaler_config': 'auto_scaler_config',
        'cluster_node_ids': 'cluster_node_ids',
        'docker_credentials_id': 'docker_credentials_id',
        'env_vars': 'env_vars',
        'image_url': 'image_url',
        'ports': 'ports',
        'resource_spec': 'resource_spec',
        'resource_spec_id': 'resource_spec_id',
        'service_account_name': 'service_account_name',
        'start_command': 'start_command',
        'volume_mount_requests': 'volume_mount_requests'
    }

    def __init__(self, auto_scaler_config=None, cluster_node_ids=None, docker_credentials_id=None, env_vars=None, image_url=None, ports=None, resource_spec=None, resource_spec_id=None, service_account_name=None, start_command=None, volume_mount_requests=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelServiceRevisionDeploymentSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._auto_scaler_config = None
        self._cluster_node_ids = None
        self._docker_credentials_id = None
        self._env_vars = None
        self._image_url = None
        self._ports = None
        self._resource_spec = None
        self._resource_spec_id = None
        self._service_account_name = None
        self._start_command = None
        self._volume_mount_requests = None
        self.discriminator = None

        if auto_scaler_config is not None:
            self.auto_scaler_config = auto_scaler_config
        if cluster_node_ids is not None:
            self.cluster_node_ids = cluster_node_ids
        self.docker_credentials_id = docker_credentials_id
        if env_vars is not None:
            self.env_vars = env_vars
        self.image_url = image_url
        if ports is not None:
            self.ports = ports
        if resource_spec is not None:
            self.resource_spec = resource_spec
        self.resource_spec_id = resource_spec_id
        if service_account_name is not None:
            self.service_account_name = service_account_name
        if start_command is not None:
            self.start_command = start_command
        if volume_mount_requests is not None:
            self.volume_mount_requests = volume_mount_requests

    @property
    def auto_scaler_config(self):
        """Gets the auto_scaler_config of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The auto_scaler_config of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: OrmAutoscalerConfig
        """
        return self._auto_scaler_config

    @auto_scaler_config.setter
    def auto_scaler_config(self, auto_scaler_config):
        """Sets the auto_scaler_config of this OrmModelServiceRevisionDeploymentSpec.


        :param auto_scaler_config: The auto_scaler_config of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type auto_scaler_config: OrmAutoscalerConfig
        """

        self._auto_scaler_config = auto_scaler_config

    @property
    def cluster_node_ids(self):
        """Gets the cluster_node_ids of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The cluster_node_ids of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: list[int]
        """
        return self._cluster_node_ids

    @cluster_node_ids.setter
    def cluster_node_ids(self, cluster_node_ids):
        """Sets the cluster_node_ids of this OrmModelServiceRevisionDeploymentSpec.


        :param cluster_node_ids: The cluster_node_ids of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type cluster_node_ids: list[int]
        """

        self._cluster_node_ids = cluster_node_ids

    @property
    def docker_credentials_id(self):
        """Gets the docker_credentials_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The docker_credentials_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: int
        """
        return self._docker_credentials_id

    @docker_credentials_id.setter
    def docker_credentials_id(self, docker_credentials_id):
        """Sets the docker_credentials_id of this OrmModelServiceRevisionDeploymentSpec.


        :param docker_credentials_id: The docker_credentials_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type docker_credentials_id: int
        """

        self._docker_credentials_id = docker_credentials_id

    @property
    def env_vars(self):
        """Gets the env_vars of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The env_vars of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: list[OrmEnvVar]
        """
        return self._env_vars

    @env_vars.setter
    def env_vars(self, env_vars):
        """Sets the env_vars of this OrmModelServiceRevisionDeploymentSpec.


        :param env_vars: The env_vars of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type env_vars: list[OrmEnvVar]
        """

        self._env_vars = env_vars

    @property
    def image_url(self):
        """Gets the image_url of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The image_url of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: str
        """
        return self._image_url

    @image_url.setter
    def image_url(self, image_url):
        """Sets the image_url of this OrmModelServiceRevisionDeploymentSpec.


        :param image_url: The image_url of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type image_url: str
        """
        if self.local_vars_configuration.client_side_validation and image_url is None:  # noqa: E501
            raise ValueError("Invalid value for `image_url`, must not be `None`")  # noqa: E501

        self._image_url = image_url

    @property
    def ports(self):
        """Gets the ports of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The ports of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: list[OrmWorkloadPort]
        """
        return self._ports

    @ports.setter
    def ports(self, ports):
        """Sets the ports of this OrmModelServiceRevisionDeploymentSpec.


        :param ports: The ports of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type ports: list[OrmWorkloadPort]
        """

        self._ports = ports

    @property
    def resource_spec(self):
        """Gets the resource_spec of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The resource_spec of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: OrmKernelResourceSpecField
        """
        return self._resource_spec

    @resource_spec.setter
    def resource_spec(self, resource_spec):
        """Sets the resource_spec of this OrmModelServiceRevisionDeploymentSpec.


        :param resource_spec: The resource_spec of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type resource_spec: OrmKernelResourceSpecField
        """

        self._resource_spec = resource_spec

    @property
    def resource_spec_id(self):
        """Gets the resource_spec_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The resource_spec_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: int
        """
        return self._resource_spec_id

    @resource_spec_id.setter
    def resource_spec_id(self, resource_spec_id):
        """Sets the resource_spec_id of this OrmModelServiceRevisionDeploymentSpec.


        :param resource_spec_id: The resource_spec_id of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type resource_spec_id: int
        """

        self._resource_spec_id = resource_spec_id

    @property
    def service_account_name(self):
        """Gets the service_account_name of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The service_account_name of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: str
        """
        return self._service_account_name

    @service_account_name.setter
    def service_account_name(self, service_account_name):
        """Sets the service_account_name of this OrmModelServiceRevisionDeploymentSpec.


        :param service_account_name: The service_account_name of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type service_account_name: str
        """

        self._service_account_name = service_account_name

    @property
    def start_command(self):
        """Gets the start_command of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The start_command of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: str
        """
        return self._start_command

    @start_command.setter
    def start_command(self, start_command):
        """Sets the start_command of this OrmModelServiceRevisionDeploymentSpec.


        :param start_command: The start_command of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type start_command: str
        """

        self._start_command = start_command

    @property
    def volume_mount_requests(self):
        """Gets the volume_mount_requests of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501


        :return: The volume_mount_requests of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :rtype: OrmVolumeMountRequests
        """
        return self._volume_mount_requests

    @volume_mount_requests.setter
    def volume_mount_requests(self, volume_mount_requests):
        """Sets the volume_mount_requests of this OrmModelServiceRevisionDeploymentSpec.


        :param volume_mount_requests: The volume_mount_requests of this OrmModelServiceRevisionDeploymentSpec.  # noqa: E501
        :type volume_mount_requests: OrmVolumeMountRequests
        """

        self._volume_mount_requests = volume_mount_requests

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelServiceRevisionDeploymentSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelServiceRevisionDeploymentSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmKernelImageEdges(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'model_service_revisions': 'list[OrmModelServiceRevision]',
        'organization': 'OrmOrganization',
        'organization_credentials': 'OrmOrganizationCredentials',
        'run_specs': 'list[OrmRunSpec]'
    }

    attribute_map = {
        'model_service_revisions': 'model_service_revisions',
        'organization': 'organization',
        'organization_credentials': 'organization_credentials',
        'run_specs': 'run_specs'
    }

    def __init__(self, model_service_revisions=None, organization=None, organization_credentials=None, run_specs=None, local_vars_configuration=None):  # noqa: E501
        """OrmKernelImageEdges - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._model_service_revisions = None
        self._organization = None
        self._organization_credentials = None
        self._run_specs = None
        self.discriminator = None

        if model_service_revisions is not None:
            self.model_service_revisions = model_service_revisions
        if organization is not None:
            self.organization = organization
        if organization_credentials is not None:
            self.organization_credentials = organization_credentials
        if run_specs is not None:
            self.run_specs = run_specs

    @property
    def model_service_revisions(self):
        """Gets the model_service_revisions of this OrmKernelImageEdges.  # noqa: E501


        :return: The model_service_revisions of this OrmKernelImageEdges.  # noqa: E501
        :rtype: list[OrmModelServiceRevision]
        """
        return self._model_service_revisions

    @model_service_revisions.setter
    def model_service_revisions(self, model_service_revisions):
        """Sets the model_service_revisions of this OrmKernelImageEdges.


        :param model_service_revisions: The model_service_revisions of this OrmKernelImageEdges.  # noqa: E501
        :type model_service_revisions: list[OrmModelServiceRevision]
        """

        self._model_service_revisions = model_service_revisions

    @property
    def organization(self):
        """Gets the organization of this OrmKernelImageEdges.  # noqa: E501


        :return: The organization of this OrmKernelImageEdges.  # noqa: E501
        :rtype: OrmOrganization
        """
        return self._organization

    @organization.setter
    def organization(self, organization):
        """Sets the organization of this OrmKernelImageEdges.


        :param organization: The organization of this OrmKernelImageEdges.  # noqa: E501
        :type organization: OrmOrganization
        """

        self._organization = organization

    @property
    def organization_credentials(self):
        """Gets the organization_credentials of this OrmKernelImageEdges.  # noqa: E501


        :return: The organization_credentials of this OrmKernelImageEdges.  # noqa: E501
        :rtype: OrmOrganizationCredentials
        """
        return self._organization_credentials

    @organization_credentials.setter
    def organization_credentials(self, organization_credentials):
        """Sets the organization_credentials of this OrmKernelImageEdges.


        :param organization_credentials: The organization_credentials of this OrmKernelImageEdges.  # noqa: E501
        :type organization_credentials: OrmOrganizationCredentials
        """

        self._organization_credentials = organization_credentials

    @property
    def run_specs(self):
        """Gets the run_specs of this OrmKernelImageEdges.  # noqa: E501


        :return: The run_specs of this OrmKernelImageEdges.  # noqa: E501
        :rtype: list[OrmRunSpec]
        """
        return self._run_specs

    @run_specs.setter
    def run_specs(self, run_specs):
        """Sets the run_specs of this OrmKernelImageEdges.


        :param run_specs: The run_specs of this OrmKernelImageEdges.  # noqa: E501
        :type run_specs: list[OrmRunSpec]
        """

        self._run_specs = run_specs

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmKernelImageEdges):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmKernelImageEdges):
            return True

        return self.to_dict() != other.to_dict()



import datetime
import time
from typing import List, TextIO

import yaml

from vessl.openapi_client import (
    InfluxdbWorkloadLog,
    ResponseRunExecutionInfo,
    ResponseRunExecutionListResponse,
)
from vessl import vessl_api
from vessl.experiment import list_experiment_logs, read_experiment_by_id
from vessl.kernel_cluster import list_clusters
from vessl.organization import _get_organization_name
from vessl.project import _get_project_name
from vessl.util.constant import LOGO, WEB_HOST, colors
from vessl.workspace import read_workspace


def read_run(
    run_id: int,
    **kwargs,
) -> ResponseRunExecutionInfo:
    """Read run in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        run_id(int): run ID.

    Example:
        ```python
        vessl.read_run(
            run_id=123,
        )
        ```
    """
    return vessl_api.run_execution_read_api(
        execution_id=run_id,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def update_run(
    run_id: int,
    description: str,
    **kwargs,
):
    """Update run in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        run_id(int): run ID.
        description(str): description of run to update.

    Example:
        ```python
        vessl.update_run(
            run_id=123,
            description="Update # of hidden layer 32->64",
        )
        ```
    """
    return vessl_api.run_execution_edit_api(
        execution_id=run_id,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        run_execution_edit_api_input={"message": description},
    )


def list_runs(
    statuses: List[str] = None,
    **kwargs,
) -> List[ResponseRunExecutionListResponse]:
    """List runs in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        statuses(List[str]): A list of status filter. Defaults to None.

    Example:
        ```python
        vessl.list_runs(
            statuses=["completed"]
        )
        ```
    """
    statuses = [",".join(statuses)] if statuses else None
    return vessl_api.run_execution_list_api(
        statuses=statuses,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    ).results


def list_run_logs(
    run_id: int,
    tail: int = 200,
    after: int = 0,
    **kwargs,
) -> List[InfluxdbWorkloadLog]:
    """List run logs in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        run_id (int): Run ID.
        tail (int): The number of lines to display from the end. Display all if
            -1. Defaults to 200.
        after (int): The number of starting lines to display from the start.
            Defaults to 0.

    Example:
        ```python
        vessl.list_run_logs(
            run_id=23,
        )
        ```
    """
    if tail == -1:
        tail = None

    return vessl_api.run_execution_logs_api(
        execution_id=run_id,
        log_limit=tail,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        log_start=after,
    ).logs


def create_run(
    yaml_file: TextIO,
    yaml_body: str,
    yaml_file_name: str,
    watch=False,
    quiet=True, # Mitigation: sdk logic which contains rich contains or print will be moved to cli
    **kwargs,
) -> ResponseRunExecutionInfo:
    """Create run in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        yaml_file (TextIO): Run ID.
        yaml_body (str): YAML body.
        yaml_file_name (str): YAML file name.
        watch (bool): Whether to watch the run.
        quiet (bool): Whether to print the log.

    Example:
        ```python
        with open(file_name, "r") as yaml_file:
            vessl.create_run(
                yaml_file=yaml_file,
                yaml_body="",
                yaml_file_name=file_name,
            )
        ```
    """
    if yaml_body == "":
        body = yaml_file.read()
    else:
        body = yaml_body
    yaml_file_name = yaml_file_name.split("/")[-1]

    wrap_str(" Launch VESSL Run 👟", "green", quiet=quiet)
    organization = _get_organization_name(**kwargs)
    project = _get_project_name(**kwargs)
    wrap_str(f"   > Organization: {organization}", "cyan", quiet=quiet)
    wrap_str(f"   > Project: {project}", "cyan", quiet=quiet)

    interactive, out_str, yaml_obj = verify_yaml(body)

    if yaml_obj == False:
        wrap_str(" YAML verification failed!", "red", quiet=quiet)
        return
    else:
        wrap_str(" YAML definition verified!", "green", quiet=quiet)
    if not quiet:
        print(out_str)
    wrap_str(f" Running: {yaml_file_name} ➡️", "green", quiet=quiet)
    # yaml_obj["run"][0]["command"] = yaml_obj["run"][0]["command"].strip()
    clean_yaml_str = yaml.dump(yaml_obj, default_flow_style=False, sort_keys=False)
    if not quiet:
        msg_box(clean_yaml_str)

    response = vessl_api.run_spec_create_from_yamlapi(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        run_spec_create_from_yamlapi_input={"yaml_spec": clean_yaml_str},
    )
    run_id = response.run_execution.id
    run_execution_response = read_run(run_id=run_id)

    link = f"{WEB_HOST}/{_get_organization_name(**kwargs)}/runs/{_get_project_name(**kwargs)}/{run_id}"
    wrap_str(
        f" Check your Run at: {link}",
        "cyan",
        quiet=False,
    )

    if not watch:
        wrap_str("   💡 Tip: Use `--watch` flag to stream logs directly to the console.", "green", quiet=quiet)
        return run_execution_response

    wrap_str(f" --watch option enabled, waiting for the Run to be scheduled...", "green", quiet=quiet)
    started = check_run_exec_started(response)
    if not started:
        return

    wrap_str(f" Showing Run logs from now:", "green", quiet=quiet)

    # fetch pod outputs
    run_finished_dt = None
    after = 0
    first_log = True
    while True:
        if read_run(run_id=run_id).status not in ["pending", "running"] and run_finished_dt is None:
            run_finished_dt = time.time()

        if run_finished_dt is not None and (time.time() - run_finished_dt) > 5:
            break

        logs = list_run_logs(
            run_id=run_id,
            before=int(time.time() - 5),
            after=after,
        )
        # do not print first log - generated while cluster was pending.
        if not first_log:
            print_logs(logs)
        else:
            first_log = False
        if len(logs) > 0:
            after = logs[-1].timestamp + 0.000001
        time.sleep(3)
    
    return run_id

def terminate_run(run_id: int, **kwargs):
    """Terminate run in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        run_id(int): Run ID.

    Example:
        ```python
        vessl.terminate_run(
            run_id=123,
        )
        ```
    """
    return vessl_api.run_execution_terminate_api(
        execution_id=run_id,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def delete_run(run_id: int, **kwargs):
    """Delete run in the default organization/project. If you want to
    override the default organization/project, then pass `organization_name` or
    `project_name` as `**kwargs`.

    Args:
        run_id(int): Run ID.

    Example:
        ```python
        vessl.delete_experiment(
            run_id=123,
        )
        ```
    """
    return vessl_api.run_execution_delete_api(
        execution_id=run_id,
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    )


def get_dt():
    dt = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    dt = f"{colors.GREY}[{dt}]{colors.ENDC}"
    return dt


def wrap_str(string, color="default", end="", quiet=False):
    if color == "cyan":
        wrapped = f"{get_dt()}{colors.OKCYAN}{string}{end}{colors.ENDC}"
    elif color == "green":
        wrapped = f"{get_dt()}{colors.OKGREEN}{string}{end}{colors.ENDC}"
    elif color == "red":
        wrapped = f"{get_dt()}{colors.FAIL}{string}{end}{colors.ENDC}"
    elif color == "warn":
        wrapped = f"{get_dt()}{colors.WARNING}{string}{end}{colors.ENDC}"
    else:
        wrapped = f"{get_dt()}{string}{end}"
    if not quiet:
        print(wrapped)
    else:
        return wrapped


def msg_box(msg):
    indent = 1
    lines = msg.split("\n")
    space = " " * indent
    width = max(map(len, lines))
    box = f'╔{"═" * (width + indent * 2)}╗\n'  # upper_border
    box += "".join([f"║{space}{line:<{width}}{space}║\n" for line in lines])
    box += f'╚{"═" * (width + indent * 2)}╝'  # lower_border
    print(box)


def print_logs(logs: List[str]):
    timezone = datetime.datetime.now().astimezone().tzinfo
    for log in logs:
        ts = datetime.datetime.fromtimestamp(log.timestamp, tz=timezone).strftime("%H:%M:%S.%f")
        message = (
            log.message.replace("\\r", "\r")
            .replace("\\n", "\n")
            .replace("\\b", "\b")
            .replace("\\t", "\t")
            .replace("\\u001b", "\u001b")
        )
        for x in message.split("\n"):
            print(f"[{ts}] {x}")


# Check different stuffs in verify_yaml.
def verify_yaml(yaml_str):
    # replace \t to double spaces
    yaml_str = yaml_str.replace("\t", "  ")
    yaml_obj = yaml.safe_load(yaml_str)
    out_str = ""

    # Step 1: Check if all necessary keys exist.
    necessary_keys = [["image"], ["resources"]]
    for keyset in necessary_keys:
        _yaml = yaml_obj
        for key in keyset:
            if key not in _yaml.keys():
                wrap_str(f" Field {key} does not exist! Please specify them in your yaml.", "red"),
                return False, False, False
            _yaml = _yaml[key]

    # Check interactive
    is_interactive = True if "interactive" in yaml_obj.keys() else False

    # Check resources
    yaml_resource = yaml_obj["resources"]
    if "cluster" in yaml_resource:
        # Collect possible cluster and gpus
        cluster = yaml_resource["cluster"]
        cluster_cands = list_clusters()
        cluster_ids = dict()
        cluster_gpus = dict()
        for e in cluster_cands:
            cluster_ids[e.name] = e.id
            cluster_gpus[e.name] = e.available_gpus

        # Verify cluster
        if cluster not in cluster_ids.keys():
            wrap_str(
                f" {cluster} cluster does not exist! Please select among {list(cluster_ids.keys())}.",
                "red",
            )
            return False, False, False
        else:
            out_str += wrap_str(f"   ✓ Cluster verified", "cyan", "\n", quiet=True)

    if is_interactive:
        out_str += wrap_str("   ✓ Mode: Interactive", "cyan", quiet=True)
    else:
        out_str += wrap_str("   - 💡 Mode: Batch", "cyan", quiet=True)

    return is_interactive, out_str, yaml_obj


def check_run_exec_started(response, quiet=True):
    run_id = response.run_execution.id

    not_started = True
    terminated = False
    while not_started and (not terminated):
        status = read_run(run_id=run_id).status
        if status != "pending":
            not_started = False
        if status in ["failed", "stopped"]:
            terminated = True

    if terminated:
        wrap_str(f" Run terminated!", "green", quiet=quiet)
        return False
    wrap_str(f"> Your Run is assigned to the cluster.", "green", quiet=quiet)

    not_started = True
    while not_started and (not terminated):
        status = read_run(run_id=run_id).status
        if status == "running":
            not_started = False
        if status in ["failed", "stopped"]:
            terminated = True
    if terminated:
        wrap_str(f" Run terminated!", "green", quiet=True)
        return False
    wrap_str(f"> Run has started!", "green", quiet=True)
    if not quiet:
        print(LOGO)
        wrap_str(f" VESSL Run has succesfully launched! 🚀", "green", quiet=True)
    return True


# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from vessl.openapi_client.configuration import Configuration


class OrmModelServiceGateway(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'created_dt': 'datetime',
        'edges': 'OrmModelServiceGatewayEdges',
        'enabled': 'bool',
        'endpoint': 'str',
        'generated_hostname': 'str',
        'id': 'int',
        'immutable_slug': 'str',
        'ingress_config': 'dict[str, object]',
        'last_error': 'str',
        'model_service_id': 'int',
        'nodeport_config': 'dict[str, object]',
        'status': 'str',
        'updated_dt': 'datetime'
    }

    attribute_map = {
        'created_dt': 'created_dt',
        'edges': 'edges',
        'enabled': 'enabled',
        'endpoint': 'endpoint',
        'generated_hostname': 'generated_hostname',
        'id': 'id',
        'immutable_slug': 'immutable_slug',
        'ingress_config': 'ingress_config',
        'last_error': 'last_error',
        'model_service_id': 'model_service_id',
        'nodeport_config': 'nodeport_config',
        'status': 'status',
        'updated_dt': 'updated_dt'
    }

    def __init__(self, created_dt=None, edges=None, enabled=None, endpoint=None, generated_hostname=None, id=None, immutable_slug=None, ingress_config=None, last_error=None, model_service_id=None, nodeport_config=None, status=None, updated_dt=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelServiceGateway - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._created_dt = None
        self._edges = None
        self._enabled = None
        self._endpoint = None
        self._generated_hostname = None
        self._id = None
        self._immutable_slug = None
        self._ingress_config = None
        self._last_error = None
        self._model_service_id = None
        self._nodeport_config = None
        self._status = None
        self._updated_dt = None
        self.discriminator = None

        if created_dt is not None:
            self.created_dt = created_dt
        if edges is not None:
            self.edges = edges
        if enabled is not None:
            self.enabled = enabled
        if endpoint is not None:
            self.endpoint = endpoint
        if generated_hostname is not None:
            self.generated_hostname = generated_hostname
        if id is not None:
            self.id = id
        if immutable_slug is not None:
            self.immutable_slug = immutable_slug
        if ingress_config is not None:
            self.ingress_config = ingress_config
        if last_error is not None:
            self.last_error = last_error
        if model_service_id is not None:
            self.model_service_id = model_service_id
        if nodeport_config is not None:
            self.nodeport_config = nodeport_config
        if status is not None:
            self.status = status
        if updated_dt is not None:
            self.updated_dt = updated_dt

    @property
    def created_dt(self):
        """Gets the created_dt of this OrmModelServiceGateway.  # noqa: E501


        :return: The created_dt of this OrmModelServiceGateway.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this OrmModelServiceGateway.


        :param created_dt: The created_dt of this OrmModelServiceGateway.  # noqa: E501
        :type created_dt: datetime
        """

        self._created_dt = created_dt

    @property
    def edges(self):
        """Gets the edges of this OrmModelServiceGateway.  # noqa: E501


        :return: The edges of this OrmModelServiceGateway.  # noqa: E501
        :rtype: OrmModelServiceGatewayEdges
        """
        return self._edges

    @edges.setter
    def edges(self, edges):
        """Sets the edges of this OrmModelServiceGateway.


        :param edges: The edges of this OrmModelServiceGateway.  # noqa: E501
        :type edges: OrmModelServiceGatewayEdges
        """

        self._edges = edges

    @property
    def enabled(self):
        """Gets the enabled of this OrmModelServiceGateway.  # noqa: E501


        :return: The enabled of this OrmModelServiceGateway.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this OrmModelServiceGateway.


        :param enabled: The enabled of this OrmModelServiceGateway.  # noqa: E501
        :type enabled: bool
        """

        self._enabled = enabled

    @property
    def endpoint(self):
        """Gets the endpoint of this OrmModelServiceGateway.  # noqa: E501


        :return: The endpoint of this OrmModelServiceGateway.  # noqa: E501
        :rtype: str
        """
        return self._endpoint

    @endpoint.setter
    def endpoint(self, endpoint):
        """Sets the endpoint of this OrmModelServiceGateway.


        :param endpoint: The endpoint of this OrmModelServiceGateway.  # noqa: E501
        :type endpoint: str
        """

        self._endpoint = endpoint

    @property
    def generated_hostname(self):
        """Gets the generated_hostname of this OrmModelServiceGateway.  # noqa: E501


        :return: The generated_hostname of this OrmModelServiceGateway.  # noqa: E501
        :rtype: str
        """
        return self._generated_hostname

    @generated_hostname.setter
    def generated_hostname(self, generated_hostname):
        """Sets the generated_hostname of this OrmModelServiceGateway.


        :param generated_hostname: The generated_hostname of this OrmModelServiceGateway.  # noqa: E501
        :type generated_hostname: str
        """

        self._generated_hostname = generated_hostname

    @property
    def id(self):
        """Gets the id of this OrmModelServiceGateway.  # noqa: E501


        :return: The id of this OrmModelServiceGateway.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OrmModelServiceGateway.


        :param id: The id of this OrmModelServiceGateway.  # noqa: E501
        :type id: int
        """

        self._id = id

    @property
    def immutable_slug(self):
        """Gets the immutable_slug of this OrmModelServiceGateway.  # noqa: E501


        :return: The immutable_slug of this OrmModelServiceGateway.  # noqa: E501
        :rtype: str
        """
        return self._immutable_slug

    @immutable_slug.setter
    def immutable_slug(self, immutable_slug):
        """Sets the immutable_slug of this OrmModelServiceGateway.


        :param immutable_slug: The immutable_slug of this OrmModelServiceGateway.  # noqa: E501
        :type immutable_slug: str
        """

        self._immutable_slug = immutable_slug

    @property
    def ingress_config(self):
        """Gets the ingress_config of this OrmModelServiceGateway.  # noqa: E501


        :return: The ingress_config of this OrmModelServiceGateway.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._ingress_config

    @ingress_config.setter
    def ingress_config(self, ingress_config):
        """Sets the ingress_config of this OrmModelServiceGateway.


        :param ingress_config: The ingress_config of this OrmModelServiceGateway.  # noqa: E501
        :type ingress_config: dict[str, object]
        """

        self._ingress_config = ingress_config

    @property
    def last_error(self):
        """Gets the last_error of this OrmModelServiceGateway.  # noqa: E501


        :return: The last_error of this OrmModelServiceGateway.  # noqa: E501
        :rtype: str
        """
        return self._last_error

    @last_error.setter
    def last_error(self, last_error):
        """Sets the last_error of this OrmModelServiceGateway.


        :param last_error: The last_error of this OrmModelServiceGateway.  # noqa: E501
        :type last_error: str
        """

        self._last_error = last_error

    @property
    def model_service_id(self):
        """Gets the model_service_id of this OrmModelServiceGateway.  # noqa: E501


        :return: The model_service_id of this OrmModelServiceGateway.  # noqa: E501
        :rtype: int
        """
        return self._model_service_id

    @model_service_id.setter
    def model_service_id(self, model_service_id):
        """Sets the model_service_id of this OrmModelServiceGateway.


        :param model_service_id: The model_service_id of this OrmModelServiceGateway.  # noqa: E501
        :type model_service_id: int
        """

        self._model_service_id = model_service_id

    @property
    def nodeport_config(self):
        """Gets the nodeport_config of this OrmModelServiceGateway.  # noqa: E501


        :return: The nodeport_config of this OrmModelServiceGateway.  # noqa: E501
        :rtype: dict[str, object]
        """
        return self._nodeport_config

    @nodeport_config.setter
    def nodeport_config(self, nodeport_config):
        """Sets the nodeport_config of this OrmModelServiceGateway.


        :param nodeport_config: The nodeport_config of this OrmModelServiceGateway.  # noqa: E501
        :type nodeport_config: dict[str, object]
        """

        self._nodeport_config = nodeport_config

    @property
    def status(self):
        """Gets the status of this OrmModelServiceGateway.  # noqa: E501


        :return: The status of this OrmModelServiceGateway.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this OrmModelServiceGateway.


        :param status: The status of this OrmModelServiceGateway.  # noqa: E501
        :type status: str
        """

        self._status = status

    @property
    def updated_dt(self):
        """Gets the updated_dt of this OrmModelServiceGateway.  # noqa: E501


        :return: The updated_dt of this OrmModelServiceGateway.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_dt

    @updated_dt.setter
    def updated_dt(self, updated_dt):
        """Sets the updated_dt of this OrmModelServiceGateway.


        :param updated_dt: The updated_dt of this OrmModelServiceGateway.  # noqa: E501
        :type updated_dt: datetime
        """

        self._updated_dt = updated_dt

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelServiceGateway):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelServiceGateway):
            return True

        return self.to_dict() != other.to_dict()



import click

from vessl.cli._base import VesslGroup
from vessl.hub import list_hub_model_tasks
from vessl.util.echo import print_info
from vessl.util.prompt import prompt_choices


@click.command(name="hub", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command(
    name="list",
    login_required=False,
)
@click.option(
    "--type",
    "-t",
    "_type",
    help="Filter by hub type. service or run.",
    default=None,
    )
def list_hub_templates(_type: str):
    
    tasks = list_hub_model_tasks(_type=_type)

    if not _type == "service":
        print_info("Run Tasks:")
        for task in tasks:
            if task.hub_type == "run":
                print_info(f"\t{task.key} ")
        print("\n")

    if not _type == "run":
        print_info("Service Tasks:")
        for task in tasks:
            if task.hub_type == "service":
                print_info(f"\t{task.key} ")


@cli.vessl_command(name="open")
@click.argument("key", type=str, default=None, required=False)
def open_task(key: str):
    if key is None:
        tasks = list_hub_model_tasks()
        selected = prompt_choices(
            "Select a task to open",
            [f"({task.hub_type})\t{task.key}" for task in tasks],
        )
        key = selected.split("\t")[1] 
    click.launch(f"https://app.vessl.ai/hub/{key}")
